%==========================================================================
% [gen_dp_claw.m] Generate CLAW elements for specified mode
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

%% --- PRELIMINARIES ---
    
% Set Laplace operator
s = tf('s');

%% --- CONFIGURE CONTROL LAWS ---

% EMF-INDI (MB)
if strcmp(options.claw.mode,'mfmb')

    % Virtual control
    K_vc   = Kp + Ki/s;

    % Angular acceleration filter           
    K_qdot = [tf(0), tf(1)]; 

    % Synchronization filter
    K_syn  = [tf(1), tf(0)];   

% EMF-INDI (SB)
elseif strcmp(options.claw.mode,'mfsb')

    % Virtual control
    K_vc   = Kp + Ki/s;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [filter_2lp_der, tf(0)];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [filter_2lp, tf(0)]; 
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), filter_2lp*H_act]; 
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), filter_2lp];  
    end
    
% EMF-INDI (HB-CF)
elseif strcmp(options.claw.mode,'mfhbf') 

    % Virtual control
    K_vc   = Kp + Ki/s;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [filter_2lp_der, 1-filter_2lp];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end              

% EMF-INDI (HB-S)
elseif strcmp(options.claw.mode,'mfhbc')

    % Virtual control
    K_vc   = Kp + Ki/s;
    
    % Angular acceleration filter
    wn_f = 100; % rad/s
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [Kc*filter_2lp_der, 1-Kc]; 

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [filter_2lp, tf(0)]; 
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), filter_2lp*H_act]; 
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), filter_2lp];  
    end                    

% EMF-INDI (HB-S #2)
elseif strcmp(options.claw.mode,'mfhbcc')

    % Virtual control
    K_vc   = Kp + Ki/s;
    
    % Angular acceleration filter
    wn_f = 100; % rad/s
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [Kc*filter_2lp_der, 1-Kc*filter_2lp]; 

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end              

% EMF-INDI (HB-SCF)
elseif strcmp(options.claw.mode,'mfhbfc')

    % Virtual control
    K_vc   = Kp + Ki/s;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);      
    
    K_qdot = [Kc*filter_2lp_der, 1-Kc*filter_2lp];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end                 

% EMF-INDI (MB; incl. 2nd-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfmb_comp')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc;

    % Angular acceleration filter           
    K_qdot = [tf(0), tf(1)]; 

    % Synchronization filter
    K_syn  = [tf(1), tf(0)]; 

% EMF-INDI (MB; incl. 1st-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfmb_comp_1')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc_1;

    % Angular acceleration filter           
    K_qdot = [tf(0), tf(1)]; 

    % Synchronization filter
    K_syn  = [tf(1), tf(0)];   

% EMF-INDI (SB; incl. 1st-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfsb_comp_1')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc_1;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [filter_2lp_der, tf(0)];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [filter_2lp, tf(0)]; 
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), filter_2lp*H_act]; 
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), filter_2lp];  
    end   

% EMF-INDI (HB-F; incl. 1st-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfhbf_comp_1')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc_1;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [filter_2lp_der, 1-filter_2lp];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end                   

% EMF-INDI (HB-S; incl. 1st-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfhbc_comp_1')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc_1;
    
    % Angular acceleration filter
    wn_f = 100; % rad/s
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);   
    
    K_qdot = [Kc*filter_2lp_der, 1-Kc]; 

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [filter_2lp, tf(0)]; 
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), filter_2lp*H_act]; 
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), filter_2lp];  
    end        

% EMF-INDI (HB-S #2; incl. 1st-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfhbcc_comp_1')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc_1;

    % Angular acceleration filter
    wn_f = 100; % rad/s
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);            
    K_qdot = [Kc*filter_2lp_der, 1-Kc*filter_2lp];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end

% EMF-INDI (HB-SCF; incl. 2nd-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfhbfc_comp')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);            
    K_qdot = [Kc*filter_2lp_der, 1-Kc*filter_2lp];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end             

% EMF-INDI (HB-SCF; incl. 1st-order vc lead-lag)
elseif strcmp(options.claw.mode,'mfhbfc_comp_1')

    % Virtual control
    K_vc   = (1 + Ki/tf('s'))*C_vc_1;

    % Angular acceleration filter
    filter_2lp_inner = feedback(wn_f/s,2*zeta_f);
    filter_2lp_der   = feedback(wn_f*filter_2lp_inner,1/s);
    filter_2lp       = feedback(wn_f*filter_2lp_inner/s,1);            
    K_qdot = [Kc*filter_2lp_der, 1-Kc*filter_2lp];

    % Synchronization filter
    if strcmp(options.claw.sync,'as')
        K_syn  = [tf(1), tf(0)];
    elseif strcmp(options.claw.sync,'am')
        K_syn  = [tf(0), H_act];
    elseif strcmp(options.claw.sync,'cc')
        K_syn  = [tf(0), pade(exp(-0.01*s),1)];
    end     
    
end     