%==========================================================================
% [plt_ag_sim.m] Plot response to 1-cos alpha-gust disturbance
%
%   Note: before using this script, an umbrella.mat file must be loaded
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

%% --- USER SETTINGS ---

close all;

% Set file destination
fsdes = '_RESULTS/SIM/';

% Set file name
ffname = 'FIG_AG_MFSB_AS_B';

% Specify colors for [NOMINAL, SAMPLED, WCASE]
clr_order = [6,3,3];

% Specify line width
LineWidthSetting = 3;

% Specify font size
font_size = 15;

% Save plot yes/no
save_plt = 1;

% Create legend yes/no
add_lgd  = 0;

% Add annotation yes/no
annotate = 0;

% Specify simulation settings
dt = 0.001;
wng = 1.3;
t = 0:dt:2*pi*2;
tg = 0:dt:2*pi/wng;
tng = t(length(tg)+1:end);
ag = 1/2*(1-cos(wng*tg));
ag = [ag, 0*tng];

%% --- GET SIMULATION DATA ---

% Create correponding nominal system description
B747_NSYS = ss(B747_USYS);

% Select model(synthesis/analysis)
UCL_PLT = A_UCL_AG;

% Sample uncertainties
UCL_SAMPLED = usample(UCL_PLT,options.plt.n_sample);

% Sample worst-case uncertainty
UCL_WC = usubs(UCL_PLT,WCU_AG_AP);

% Get nominal system
NCL = ss(UCL_PLT);
    
% Get responses
SSYS_nz_ag     = UCL_SAMPLED(out_idx_nz,1);
WSYS_nz_ag     = UCL_WC(out_idx_nz,1);
NSYS_nz_ag     = NCL(out_idx_nz,1);

SSYS_q_ag     = 180/pi*UCL_SAMPLED(out_idx_q,1);
WSYS_q_ag     = 180/pi*UCL_WC(out_idx_q,1);
NSYS_q_ag     = 180/pi*NCL(out_idx_q,1);

%% --- PLOT ---

% Set color theme
clr_theme = [ [0, 0.4470, 0.7410]; ...
            [0.8500, 0.3250, 0.0980]; ...
            [0.4660, 0.6740, 0.1880]; ...
            [0.4940, 0.1840, 0.5560]; ...  
            [0.6350, 0.0780, 0.1840]; ...
            [0.75, 0.75, 0.75]; ...
            [0.75, 0.75, 0]; ...
            [0, 0.75, 0.75]; ...
            [0.9290, 0.6940, 0.1250]];

% Get files in specified directory
dir_lst = dir; dir_lst = dir_lst(3:end);

% Create figure [left bottom width height]
FIG = figure; FIG.Position = [300 59 560 600];

% Simulate and plot response
for i = 1:options.plt.n_sample

    if i == 1
        sp1 = subplot(2,1,1);
        sp1.Position = sp1.Position + [0 -0.03 0.05 0.08];
    
        % Create textbox
        if annotate
            annotation(FIG,'textbox',...
                [0.566 0.882352941176471 0.32 0.0631470588235294],...
                'String',strcat('$\gamma_{\alpha_g \rightarrow z_{mf}} = '...
                ,num2str(round(WCG_AG_AP.UpperBound,2)),'$'),'Interpreter','latex',...
                'FontSize',20,'BackgroundColor',[1 1 1]);
        end
    end
    
    if i <options.plt.n_sample        
        
        if i == 1
            [y_q_ag_ba,~] = lsim(B747_NSYS(1,4),ag,t); 
            [y_q_ds,~] = lsim(NSYS_q_ag,ag,t);
            [y_q_ds_wc,~] = lsim(WSYS_q_ag,ag,t);   
        
            plot(sp1,t,180/pi*y_q_ag_ba,'k--','LineWidth',LineWidthSetting);
            grid on; hold on;
            plot(sp1,t,y_q_ds,'color',clr_theme(clr_order(2),:),'LineStyle','-','LineWidth',LineWidthSetting);
            plot(sp1,t,y_q_ds_wc,'color',clr_theme(clr_order(3),:),'LineStyle','-.','LineWidth',LineWidthSetting);
        end
        
        [y_q_ds,~] = lsim(SSYS_q_ag(:,:,i,1),ag,t);
       
        plot(sp1,t,y_q_ds,'color',clr_theme(clr_order(1),:),'LineStyle','-','LineWidth',LineWidthSetting);
        
    else
        
        [y_q_ds,~] = lsim(NSYS_q_ag,ag,t);     
        
        plot(sp1,t,180/pi*y_q_ag_ba,'k--','LineWidth',LineWidthSetting);
        grid on; hold on;
        plot(sp1,t,y_q_ds_wc,'color',clr_theme(clr_order(3),:),'LineStyle','-.','LineWidth',LineWidthSetting);
        plot(sp1,t,y_q_ds,'color',clr_theme(clr_order(2),:),'LineStyle','-','LineWidth',LineWidthSetting);
        
        xlim(sp1,[0,12])
        ylim(sp1,[-1.5,1.5]);
        ylabel(sp1,'Pitch rate (deg/s)','Interpreter','latex','FontSize',font_size);
        set(sp1,'TickLabelInterpreter','latex','FontSize',font_size)
        set(sp1,'XTickLabel',[])
        
    end

    % Add legend
    if add_lgd
        legend(sp1, 'Bare airframe', 'Nominal', 'Worst-case', 'Sampled', ...
            'Orientation','horizontal','NumColumns',2, ...
            'Interpreter','latex','Location','southeast','FontSize',font_size);
    end
        
    if i == 1
        sp2 = subplot(2,1,2);
        sp2.Position = sp2.Position + [0 -0.02 0.05 0.08];
    end
    
    if i < options.plt.n_sample         
        
        if i == 1
            [y_nz_ds,~] = lsim(NSYS_nz_ag,ag,t);
            [y_nz_ag_ba,~] = lsim(B747_NSYS(3,4),ag,t);  
            [y_nz_ds_wc,~] = lsim(WSYS_nz_ag,ag,t);
         
            plot(sp2,t,y_nz_ag_ba,'k--','LineWidth',LineWidthSetting);
            grid on; hold on;
            plot(sp2,t,y_nz_ds,'color',clr_theme(clr_order(2),:),'LineStyle','-','LineWidth',LineWidthSetting);
        end 
        
        [y_nz_ds,~] = lsim(SSYS_nz_ag(:,:,i,1),ag,t);
       
        plot(sp2,t,y_nz_ds,'color',clr_theme(clr_order(1),:),'LineStyle','-','LineWidth',LineWidthSetting);
        grid on; hold on;
        
    else
        
        [y_nz_ds,~] = lsim(NSYS_nz_ag,ag,t);
        
        plot(sp2,t,y_nz_ag_ba,'k--','LineWidth',LineWidthSetting);
        grid on; hold on;
        plot(sp2,t,y_nz_ds_wc,'color',clr_theme(clr_order(3),:),'LineStyle','-.','LineWidth',LineWidthSetting);
        plot(sp2,t,y_nz_ds,'color',clr_theme(clr_order(2),:),'LineStyle','-','LineWidth',LineWidthSetting);
        
        xlim(sp2,[0,12])
        ylim(sp2,[-0.2,0.5]);
        xlabel(sp2,'Time (s)','Interpreter','latex','FontSize',font_size);
        ylabel(sp2,'Normal acceleration (g)','Interpreter','latex','FontSize',font_size);
        set(sp2,'TickLabelInterpreter','latex','FontSize',font_size)
        
    end        

end

%% Save plot

FIG.Renderer = 'painters';

if save_plt
    saveas(FIG,strcat(fsdes,ffname,'.fig'));
    saveas(FIG,strcat(fsdes,ffname,'.png'));
    saveas(FIG,strcat(fsdes,ffname,'.emf'));
    saveas(FIG,strcat(fsdes,ffname),'epsc');
end